@@ -4,9 +4,9 @@
  
  Copyright (C) 1997-now
  Core Developers:
- Sergei L Kosakovsky Pond (spond@ucsd.edu)
+ Sergei L Kosakovsky Pond (sergeilkp@icloud.com)
  Art FY Poon    (apoon@cfenet.ubc.ca)
- Steven Weaver (sweaver@ucsd.edu)
+ Steven Weaver (sweaver@temple.edu)
  
  Module Developers:
  Lance Hepler (nlhepler@gmail.com)
@@ -43,6 +43,7 @@
 #include      "likefunc.h"
 #include      "bayesgraph.h"
 #include      "scfg.h"
+#include      "function_templates.h"
 
 #if defined __MAC__ || defined __WINDOZE__ || defined __HYPHY_GTK__
     #include "HYConsoleWindow.h"
@@ -55,6 +56,10 @@
 #include "hyphy_qt_helpers.h"
 #endif
 
+#include "global_object_lists.h"
+
+using namespace hyphy_global_objects;
+
 _List       openFileHandlesBackend;
 
 _AVLListX   openFileHandles     (&openFileHandlesBackend);
@@ -77,7 +82,7 @@ bool      _ElementaryCommand::HandleHarvestFrequencies (_ExecutionList& currentP
 
 
     long       objectType = HY_BL_DATASET|HY_BL_DATASET_FILTER;
-    BaseRef    sourceObject = _HYRetrieveBLObjectByName (dataID, objectType,nil,false);
+    BaseRefConst    sourceObject = _HYRetrieveBLObjectByName (dataID, objectType,nil,false);
     
     long      unit      = ProcessNumericArgument((_String*)parameters(2),currentProgram.nameSpacePrefix),
               posspec   = ProcessNumericArgument((_String*)parameters(4),currentProgram.nameSpacePrefix),
@@ -99,7 +104,7 @@ bool      _ElementaryCommand::HandleHarvestFrequencies (_ExecutionList& currentP
             hSpecs = *(_String*)parameters(6);
         }
         
-        _DataSet * dataset = (_DataSet*)sourceObject;
+        _DataSet const * dataset = (_DataSet const*)sourceObject;
         _SimpleList     hL, vL;
         dataset->ProcessPartition (hSpecs,hL,false);
         dataset->ProcessPartition (vSpecs,vL,true);
@@ -107,13 +112,13 @@ bool      _ElementaryCommand::HandleHarvestFrequencies (_ExecutionList& currentP
         receptacle = dataset->HarvestFrequencies(unit,atom,posspec,hL, vL,cghf>0.5);
     } else { // harvest from a DataSetFilter
         if (objectType == HY_BL_DATASET_FILTER) {
-            receptacle = ((_DataSetFilter*)sourceObject)->HarvestFrequencies(unit,atom,posspec,cghf>0.5);
+            receptacle = ((_DataSetFilter const*)sourceObject)->HarvestFrequencies(unit,atom,posspec,cghf>0.5);
         } else {
             errMsg = _String ("'") & dataID & "' is neither a DataSet nor a DataSetFilter";
         }
     }
     
-    SetStatusLine           (empty);
+    SetStatusLine           (emptyString);
     
     if (errMsg.sLength || receptacle == nil) {
         DeleteObject (receptacle);
@@ -243,11 +248,8 @@ bool      _ElementaryCommand::HandleComputeLFFunction (_ExecutionList& currentPr
             WarnError (_String("Please call LFCompute (lf_id, ")&lfStartCompute&") before evaluating the likelihood function");
             return false;
         } else {
-            _Variable* rec = CheckReceptacleCommandID (&AppendContainerName(*arg2,currentProgram.nameSpacePrefix), HY_HBL_COMMAND_LFCOMPUTE,true);
-             if (!rec) {
-                return false;
-            }
-            rec->SetValue(new _Constant (lf->Compute()),false);
+            return CheckReceptacleCommandIDAndStore(&AppendContainerName(*arg2,currentProgram.nameSpacePrefix), HY_HBL_COMMAND_LFCOMPUTE, HY_HBL_COMMAND_LFCOMPUTE, new _Constant (lf->Compute()), false);
+                                             
         }
     }
 
@@ -285,7 +287,7 @@ bool      _ElementaryCommand::HandleSelectTemplateModel (_ExecutionList& current
         long                dataDimension   = thisDF->GetDimension(),
                             unitLength      = thisDF->GetUnitLength();
 
-        _TranslationTable*  thisTT = thisDF->GetData()->GetTT();
+        _TranslationTable const*  thisTT = thisDF->GetData()->GetTT();
         
         if (unitLength==1) {
             if (thisTT->IsStandardNucleotide()) {
@@ -294,12 +296,14 @@ bool      _ElementaryCommand::HandleSelectTemplateModel (_ExecutionList& current
                 dataType = "aminoacid";
             }
         } else {
-            if (thisTT->IsStandardNucleotide())
+          if (thisTT->IsStandardNucleotide()) {
                 if (unitLength==3) {
                     dataType = "codon";
-                } else if (unitLength==2) {
-                    dataType = "dinucleotide";
+                } else {
+                    if (unitLength==2)
+                      dataType = "dinucleotide";
                 }
+          }
         }
 
         if (!dataType.sLength) {
@@ -484,7 +488,7 @@ bool      _ElementaryCommand::HandleSetParameter (_ExecutionList& currentProgram
     long objectIndex,
          typeFlag    = HY_BL_ANY;
     
-    BaseRef theObject      = _HYRetrieveBLObjectByName (nmspc, typeFlag, &objectIndex);
+    BaseRef theObject      = _HYRetrieveBLObjectByNameMutable (nmspc, typeFlag, &objectIndex);
     
     switch (typeFlag)
     {
@@ -613,7 +617,7 @@ bool      _ElementaryCommand::HandleSetParameter (_ExecutionList& currentProgram
                 _LikelihoodFunction * lkf = (_LikelihoodFunction *) theObject;
                 currentArgument = (_String*)parameters(1);
                 long g = ProcessNumericArgument(currentArgument,currentProgram.nameSpacePrefix);
-                if (g < 0 || g >= lkf->GetIndependentVars().lLength) {
+                if (lkf->GetIndependentVars().Map (g) < 0L) {
                     currentProgram.ReportAnExecutionError (*currentArgument & " (=" & g & ") is not a valid parameter index");
                     return false;
                 }
@@ -627,30 +631,31 @@ bool      _ElementaryCommand::HandleSetParameter (_ExecutionList& currentProgram
         case HY_BL_DATASET:
         case HY_BL_DATASET_FILTER: {
             _DataSet * ds = nil;
+          
             
-            long f  = ProcessNumericArgument ((_String*)parameters(1),currentProgram.nameSpacePrefix);
+            long sequence_index  = ProcessNumericArgument ((_String*)parameters(1),currentProgram.nameSpacePrefix);
             if (typeFlag == HY_BL_DATASET) {
                 ds = (_DataSet*) theObject;
             }
             else {
                 _DataSetFilter *dsf = (_DataSetFilter*)theObject;
                 ds = dsf->GetData ();
-                if (f >= 0 && f < dsf->theNodeMap.lLength){
-                    f  = dsf->theNodeMap.lData[f];
-                }
-                else
-                    f = -1;
+                sequence_index = dsf->theNodeMap.Map (sequence_index);
+            }
+          
+            if (typeFlag == HY_BL_DATASET_FILTER) {
+              ReleaseDataFilterLock(objectIndex);
             }
-            
         
-            _List*  dsNames = &ds->GetNames();
-            
-            if (f<0 || f>=dsNames->lLength) {
-                currentProgram.ReportAnExecutionError (*((_String*)parameters(1)) & " (=" & f & ") is not a valid sequence index");
-                return false;
-            }
-
-            dsNames->Replace(f, new _String(ProcessLiteralArgument ((_String*)parameters(2),currentProgram.nameSpacePrefix)), false);
+          
+          _String * sequence_name = new _String(ProcessLiteralArgument ((_String*)parameters(2),currentProgram.nameSpacePrefix));
+          
+          if (! ds->SetSequenceName (sequence_index, sequence_name)) {
+            delete sequence_name;
+            currentProgram.ReportAnExecutionError (*((_String*)parameters(1)) & " (=" & sequence_index & ") is not a valid sequence index");
+            return false;
+           
+          }
         } // end data set and data set filter
         break; 
         // Dataset and Datasetfilter
@@ -662,7 +667,7 @@ bool      _ElementaryCommand::HandleSetParameter (_ExecutionList& currentProgram
                 if (*((_String*)parameters(1)) == _String("MODEL")) {
                     _String modelName = AppendContainerName(*((_String*)parameters(2)),currentProgram.nameSpacePrefix);
                     long modelType = HY_BL_MODEL, modelIndex;
-                    BaseRef modelObject      = _HYRetrieveBLObjectByName (modelName, modelType, &modelIndex, true);
+                    BaseRef modelObject      = _HYRetrieveBLObjectByNameMutable (modelName, modelType, &modelIndex, true);
                     if (modelObject) {
                         _VariableContainer * parentTree = treeNode->ParentTree();
                         if (!parentTree) {
@@ -670,12 +675,14 @@ bool      _ElementaryCommand::HandleSetParameter (_ExecutionList& currentProgram
                             return false;
                         }
                         long pID, lfID = ((_TheTree*)parentTree->Compute())->IsLinkedToALF(pID);
-                        if (lfID>=0){
-                             currentProgram.ReportAnExecutionError ((*parentTree->GetName()) & " is linked to a likelihood function (" & *_HBLObjectNameByType (HY_BL_LIKELIHOOD_FUNCTION, lfID) &") and cannot be modified ");
-                             return false;
-                        }
-                        
+                      
                         treeNode->ReplaceModel (modelName, parentTree);
+                      
+                        if (lfID>=0){
+                          ((_LikelihoodFunction*)likeFuncList(lfID))->Rebuild(true);
+                          //currentProgram.ReportAnExecutionError ((*parentTree->GetName()) & " is linked to a likelihood function (" & *GetObjectNameByType (HY_BL_LIKELIHOOD_FUNCTION, lfID) &") and cannot be modified ");
+                          //  return false;
+                        }
                         break;
                     }
                     else {
@@ -743,9 +750,32 @@ bool      _ElementaryCommand::HandleRequireVersion(_ExecutionList& currentProgra
     currentProgram.currentCommand++;
     _String theVersion = ProcessLiteralArgument ((_String*)parameters (0),currentProgram.nameSpacePrefix);
 
-    if (__KERNEL__VERSION__.toNum() < theVersion.toNum()) {
-        currentProgram.ReportAnExecutionError (_String ("Current batch file requires at least version :")& theVersion &" of HyPhy. Please download an updated version from http://www.hyphy.org and try again.");
-        return false;
+    _List local_version    = __HYPHY__VERSION__.Tokenize ("."),
+          required_version = theVersion.Tokenize(".");
+  
+    try {
+    
+      unsigned long const  upper_bound = MIN (local_version.countitems(), required_version.countitems());
+    
+    
+      for (unsigned long i = 0UL; i < upper_bound; i++) {
+        _Parameter local_number = ((_String*)local_version.GetItem(i))->toNum();
+        _Parameter required_number = ((_String*)required_version.GetItem(i))->toNum();
+        
+        if (local_number > required_number) {
+          return true;
+        }
+        if (local_number < required_number) {
+          throw (0);
+        }
+      }
+    
+      if (required_version.countitems() > upper_bound) {
+          return true;
+      }
+    } catch (int ) {
+      currentProgram.ReportAnExecutionError (_String ("Current batch file requires at least version :")& theVersion &" of HyPhy. Please download an updated version from http://www.hyphy.org and try again.");
+      return false;
     }
     return true;
 }
@@ -757,7 +787,7 @@ bool      _ElementaryCommand::HandleDeleteObject(_ExecutionList& currentProgram)
     for (unsigned long objCount = 0; objCount < parameters.lLength; objCount++) {
         long       objectType = HY_BL_LIKELIHOOD_FUNCTION,
                    f = -1;
-        BaseRef    sourceObject = _HYRetrieveBLObjectByName (AppendContainerName(*(_String*)parameters(objCount),currentProgram.nameSpacePrefix), objectType,&f,false);
+        BaseRef    sourceObject = _HYRetrieveBLObjectByNameMutable (AppendContainerName(*(_String*)parameters(objCount),currentProgram.nameSpacePrefix), objectType,&f,false);
 
         if  (sourceObject) {
             KillLFRecord (f,true);
@@ -808,7 +838,7 @@ bool      _ElementaryCommand::HandleMolecularClock(_ExecutionList& currentProgra
     } else {
         treeName    = *theObject->GetName();
         theTree     = (_TheTree*)theObject;
-        theBaseNode = empty;
+        theBaseNode = emptyString;
     }
     
     theTree->MolecularClock(theBaseNode,parameters);
@@ -883,7 +913,7 @@ bool      _ElementaryCommand::HandleGetString (_ExecutionList& currentProgram){
     if (f >=0 ) {
         f = _HY_GetStringGlobalTypes.GetXtra (f);
     }
-
+  
     switch (f) {
 
         case HY_BL_LIKELIHOOD_FUNCTION: // LikelihoodFunction
@@ -891,7 +921,7 @@ bool      _ElementaryCommand::HandleGetString (_ExecutionList& currentProgram){
         case HY_BL_DATASET_FILTER:
         case HY_BL_SCFG:
         case HY_BL_BGM: {
-            result = (_String*)_HBLObjectNameByType(f,sID);
+            result = (_String*)GetObjectNameByType(f,sID);
             if (result) {
                 result = (_String*) result->makeDynamic();
 				//ReportWarning(_String((const char*)"In HandleGetString(): ") & result);
@@ -900,11 +930,11 @@ bool      _ElementaryCommand::HandleGetString (_ExecutionList& currentProgram){
         }
            
         case HY_BL_HBL_FUNCTION: // UserFunction
-            result = (_String*)_HBLObjectNameByType(HY_BL_HBL_FUNCTION,sID);
+            result = (_String*)GetObjectNameByType(HY_BL_HBL_FUNCTION,sID);
             if (result) {
-                _AssociativeList * resAVL = (_AssociativeList *)checkPointer(new _AssociativeList);
+                _AssociativeList * resAVL = new _AssociativeList;
                 resAVL->MStore ("ID", new _FString (*result), false);
-                resAVL->MStore ("Arguments", new _Matrix(*(_List*)batchLanguageFunctionParameterLists(sID)), false);
+                resAVL->MStore ("Arguments", new _Matrix(GetBFFunctionArgumentList(sID)), false);
                 theReceptacle->SetValue (resAVL,false);
                 return true;
             } 
@@ -922,20 +952,22 @@ bool      _ElementaryCommand::HandleGetString (_ExecutionList& currentProgram){
 
         default: { // everything else...
             // decide what kind of object current argument represents
-            
+          
+          
+          
             _String *currentArgument = (_String*)parameters(1),
                     nmspaced       = AppendContainerName(*currentArgument,currentProgram.nameSpacePrefix);
             long    typeFlag       = HY_BL_ANY,
                     index          = -1;
                     
-            BaseRef theObject      = _HYRetrieveBLObjectByName (nmspaced, typeFlag, &index);
+            BaseRefConst theObject      = _HYRetrieveBLObjectByName (nmspaced, typeFlag, &index);
 
             if (theObject) {
                 switch (typeFlag) {
                 case HY_BL_DATASET: {
-                    _DataSet* dataSetObject = (_DataSet*)theObject;
+                    _DataSet const* dataSetObject = (_DataSet const*)theObject;
                     if (sID>=0 && sID<dataSetObject->NoOfSpecies()) {
-                        result = (_String*)(dataSetObject->GetNames())(sID)->makeDynamic();
+                        result = (_String*)(dataSetObject->GetNames().GetItem(sID))->makeDynamic();
                     } else {
                         if (sID < 0) {
                             theReceptacle->SetValue (new _Matrix (dataSetObject->GetNames()), false);
@@ -945,16 +977,17 @@ bool      _ElementaryCommand::HandleGetString (_ExecutionList& currentProgram){
                     break;
                 }
                 case HY_BL_DATASET_FILTER: {
-                    _DataSetFilter* dataSetFilterObject = (_DataSetFilter*)theObject;
+                    _DataSetFilter const* dataSetFilterObject = (_DataSetFilter const*)theObject;
 
                     if (sID >=0 && sID<dataSetFilterObject->NumberSpecies()) {
-                        result = (_String*)(dataSetFilterObject->GetData()->GetNames())(dataSetFilterObject->theNodeMap(sID))->makeDynamic();
+                        result = (_String*)dataSetFilterObject->GetData()->GetNames().GetItem(dataSetFilterObject->theNodeMap.Element(sID))->makeDynamic();
                     } else {
                         if (sID < 0) {
-                            _List filterSeqNames,
-                                  *originalNames = &dataSetFilterObject->GetData()->GetNames();
+                            _List filterSeqNames;
+                            _List const * originalNames = &dataSetFilterObject->GetData()->GetNames();
+                          
                             for (long seqID=0; seqID<dataSetFilterObject->NumberSpecies(); seqID++) {
-                                filterSeqNames << (*originalNames)(dataSetFilterObject->theNodeMap(seqID));
+                                filterSeqNames << originalNames->GetItem (dataSetFilterObject->theNodeMap.Element (seqID));
                             }
                             theReceptacle->SetValue (new _Matrix (filterSeqNames), false);
                             return true;
@@ -964,7 +997,7 @@ bool      _ElementaryCommand::HandleGetString (_ExecutionList& currentProgram){
                 }
                 case HY_BL_BGM: {
                     //ReportWarning(_String("In HandleGetString() for case HY_BL_BGM"));
-					_BayesianGraphicalModel * this_bgm      = (_BayesianGraphicalModel *) theObject;
+                    _BayesianGraphicalModel * this_bgm      = (_BayesianGraphicalModel *) theObject;
 
                     switch (sID) {
                         case HY_HBL_GET_STRING_BGM_SCORE: {   // return associative list containing node score cache
@@ -1066,9 +1099,9 @@ bool      _ElementaryCommand::HandleGetString (_ExecutionList& currentProgram){
                 }
                 case HY_BL_HBL_FUNCTION: {
                     _AssociativeList * resAVL = (_AssociativeList *)checkPointer(new _AssociativeList);
-                    resAVL->MStore ("ID", new _FString (*_HBLObjectNameByType (HY_BL_HBL_FUNCTION, index, false)), false);
-                    resAVL->MStore ("Arguments", new _Matrix(*(_List*)batchLanguageFunctionParameterLists(index)), false);
-                    resAVL->MStore("Body", new _FString (((_ExecutionList*)batchLanguageFunctions(index))->sourceText,false),false);
+                    resAVL->MStore ("ID", new _FString (*GetObjectNameByType (HY_BL_HBL_FUNCTION, index, false)), false);
+                    resAVL->MStore ("Arguments", new _Matrix(GetBFFunctionArgumentList(index)), false);
+                    resAVL->MStore ("Body", new _FString (GetBFFunctionBody(index).sourceText,false),false);
                     theReceptacle->SetValue (resAVL,false);
                     return true;
                 }
@@ -1078,29 +1111,32 @@ bool      _ElementaryCommand::HandleGetString (_ExecutionList& currentProgram){
             if (currentArgument->Equal(&versionString)) {
                 if (sID > 1.5)
 #ifdef __HEADLESS__
-                    result = new _String(_String ("Library version ") & __KERNEL__VERSION__);
+                    result = new _String(_String ("Library version ") & __HYPHY__VERSION__);
 #else
 #ifdef __MAC__
-                    result = new _String(_String("Macintosh ") & __KERNEL__VERSION__);
+                    result = new _String(_String("Macintosh ") & __HYPHY__VERSION__);
 #else
 #ifdef __WINDOZE__
-                    result = new _String(_String("Windows ") & __KERNEL__VERSION__);
+                    result = new _String(_String("Windows ") & __HYPHY__VERSION__);
 #else
-                    result = new _String(_String("Source ") & __KERNEL__VERSION__);
+                    result = new _String(_String("Source ") & __HYPHY__VERSION__);
 #endif
 #endif
 #endif
                     else if (sID > 0.5) {
                         result = new _String(GetVersionString());
                     } else {
-                        result = new _String(__KERNEL__VERSION__);
+                        result = new _String(__HYPHY__VERSION__);
                     }
                 } else if (currentArgument->Equal(&timeStamp)) {
                     result = new _String(GetTimeStamp(sID < 0.5));
+                } else if (currentArgument->Equal(&listLoadedLibraries)) {
+                  theReceptacle->SetValue (new _Matrix (loadedLibraryPaths.Keys()));
+                  return true;
                 } else {
-                  _Variable* theVar = FetchVar(LocateVarByName (*currentArgument));
+                  _Variable* theVar = FetchVar(LocateVarByName (nmspaced));
                   if (theVar) {
-                    if (theVar->IsIndependent()) {
+                    if (theVar->IsIndependent() && sID != -3) {
                       result = (_String*)theVar->toStr();
                     } else {
                       if (sID == -1){
@@ -1120,14 +1156,31 @@ bool      _ElementaryCommand::HandleGetString (_ExecutionList& currentProgram){
                       }
                       
                       else {  // formula string
-                        _Matrix * formula_matrix = (sID2 >= 0 && theVar->ObjectClass() == MATRIX) ? (_Matrix*)theVar->GetValue () : nil;
-                        if (formula_matrix) {
-                         _Formula* cell = formula_matrix->GetFormula(sID, sID2);
-                         if (cell) {
-                            result = (_String*) cell->toStr();
+                        
+                        if (sID == -3) {
+                          _String local, global;
+                          _SimpleList var_index;
+                          var_index << theVar->GetAVariable ();
+                          if (theVar->IsIndependent()) {
+                            //printf ("ExportIndVariables\n");
+                            ExportIndVariables (global, local, &var_index);
+                          } else {
+                            //printf ("ExportDepVariables\n");
+                            ExportDepVariables(global, local, &var_index);
                           }
+                          result = new _String (128L, true);
+                          (*result) << global << local << '\n';
+                          result->Finalize();
                         } else {
-                          result = (_String*)theVar->GetFormulaString ();
+                          _Matrix * formula_matrix = (sID2 >= 0 && theVar->ObjectClass() == MATRIX) ? (_Matrix*)theVar->GetValue () : nil;
+                          if (formula_matrix) {
+                           _Formula* cell = formula_matrix->GetFormula(sID, sID2);
+                           if (cell) {
+                              result = (_String*) cell->toStr();
+                            }
+                          } else {
+                            result = (_String*)theVar->GetFormulaString ();
+                          }
                         }
                       }
                     }
@@ -1166,18 +1219,18 @@ bool      _ElementaryCommand::HandleExport(_ExecutionList& currentProgram){
     _String objectID (currentProgram.AddNameSpaceToID(*(_String*)parameters(1))),
             arg1 (currentProgram.AddNameSpaceToID(*(_String*)parameters(0))),
             errMsg;
-    
+  
     _Variable * theReceptacle = CheckReceptacleCommandID (&AppendContainerName(arg1,currentProgram.nameSpacePrefix),HY_HBL_COMMAND_EXPORT, true, false, &currentProgram);
     if (!theReceptacle) {
         return false;
     }    
-
-    _FString        * outLF = new _FString (new _String (8192L,1));
-    checkPointer    (outLF);
-    long typeFlag = HY_BL_MODEL | HY_BL_LIKELIHOOD_FUNCTION | HY_BL_DATASET_FILTER,
+ 
+    _FString        * outLF = new _FString (new _String (8192UL,1));
+ 
+    long typeFlag = HY_BL_MODEL | HY_BL_LIKELIHOOD_FUNCTION | HY_BL_DATASET_FILTER | HY_BL_HBL_FUNCTION,
              index;
         
-    BaseRef objectToExport = _HYRetrieveBLObjectByName (objectID, typeFlag, &index);
+    BaseRef objectToExport = _HYRetrieveBLObjectByNameMutable (objectID, typeFlag, &index);
     if (! objectToExport) {
         errMsg = _String ("'") & objectID & "' is not a supported type";
     } else {
@@ -1190,7 +1243,8 @@ bool      _ElementaryCommand::HandleExport(_ExecutionList& currentProgram){
             case HY_BL_DATASET_FILTER: {
                 outLF->theString->Finalize();
                 DeleteObject (outLF->theString);
-                checkPointer (outLF->theString = new _String ((_String*)((_DataSetFilter*)objectToExport)->toStr()));
+                outLF->theString = new _String ((_String*)((_DataSetFilter*)objectToExport)->toStr());
+                ReleaseDataFilterLock(index);
                 break;
             }
             case HY_BL_MODEL: {
@@ -1198,7 +1252,12 @@ bool      _ElementaryCommand::HandleExport(_ExecutionList& currentProgram){
                 outLF->theString->Finalize();
                 break;
             }
-                
+            case HY_BL_HBL_FUNCTION: {
+                (*outLF->theString) << ExportBFFunction (index);
+                outLF->theString->Finalize();
+                break;
+            }
+            
         }
     }
 
@@ -1303,7 +1362,7 @@ bool      _ElementaryCommand::HandleFprintf (_ExecutionList& currentProgram)
             }
         }
         
-        checkParameter (printDigitsSpec,printDigits,0);
+        checkParameter (printDigitsSpec,printDigits,0L);
         
         if (!print_to_stdout) {
             fnm = *targetName;
@@ -1379,9 +1438,10 @@ bool      _ElementaryCommand::HandleFprintf (_ExecutionList& currentProgram)
                 
                 
                 if (thePrintObject == nil) {
-                    long typeFlag = HY_BL_ANY;
-                    
-                    thePrintObject = _HYRetrieveBLObjectByName (nmspace, typeFlag);
+                    long typeFlag = HY_BL_ANY,
+                         index;
+                  
+                    thePrintObject = _HYRetrieveBLObjectByNameMutable (nmspace, typeFlag, &index);
                     
                     if (!thePrintObject) {
                         _String argCopy = *varname,
@@ -1396,6 +1456,10 @@ bool      _ElementaryCommand::HandleFprintf (_ExecutionList& currentProgram)
                             else
                                 throw (_String ("Argument ") & i & " is not a simple expression");
                         }
+                    } else {
+                      if (typeFlag == HY_BL_DATASET_FILTER) {
+                        ReleaseDataFilterLock(index);
+                      }
                     }
                 }
             }
