@@ -4,9 +4,9 @@
  
  Copyright (C) 1997-now
  Core Developers:
- Sergei L Kosakovsky Pond (spond@ucsd.edu)
+ Sergei L Kosakovsky Pond (sergeilkp@icloud.com)
  Art FY Poon    (apoon@cfenet.ubc.ca)
- Steven Weaver (sweaver@ucsd.edu)
+ Steven Weaver (sweaver@temple.edu)
  
  Module Developers:
  Lance Hepler (nlhepler@gmail.com)
@@ -39,6 +39,7 @@
 
 
 #include "bayesgraph.h"
+#include "function_templates.h"
 
 extern _String      _HYBgm_IMPUTE_MAXSTEPS,
        _HYBgm_IMPUTE_BURNIN,
@@ -788,17 +789,18 @@ _Parameter _BayesianGraphicalModel::ImputeDiscreteNodeScore (long node_id, _Simp
     
 	
 	_Parameter      log_score           = 0,
-					denom, this_prob,
-					impute_maxsteps, impute_burnin, impute_samples; // HBL settings
+  denom, this_prob;
+  
+	long				impute_maxsteps, impute_burnin, impute_samples; // HBL settings
     
 	double			urn;	// uniform random number
 	
 	
 	
 	// set Gibbs sampler parameters from batch language definitions
-    checkParameter (_HYBgm_IMPUTE_MAXSTEPS, impute_maxsteps, 0);
-    checkParameter (_HYBgm_IMPUTE_BURNIN, impute_burnin, 0);
-    checkParameter (_HYBgm_IMPUTE_SAMPLES, impute_samples, 0);
+    checkParameter (_HYBgm_IMPUTE_MAXSTEPS, impute_maxsteps, 0L);
+    checkParameter (_HYBgm_IMPUTE_BURNIN, impute_burnin, 0L);
+    checkParameter (_HYBgm_IMPUTE_SAMPLES, impute_samples, 0L);
 	
     if (impute_maxsteps <= 0 || impute_burnin < 0 || impute_samples <= 0 || impute_samples > impute_maxsteps) {
         WarnError (_String("ERROR: Invalid IMPUTE setting(s) in ImputeNodeScore()"));
@@ -900,10 +902,10 @@ _Parameter _BayesianGraphicalModel::ImputeDiscreteNodeScore (long node_id, _Simp
 	
 	
     // initialize missing entries to random assignments based on observed cases or prior info
-    for (long fnode, row, col, missing_idx = 0; missing_idx < is_missing.lLength; missing_idx++) {
+    for (long row, col, missing_idx = 0; missing_idx < is_missing.lLength; missing_idx++) {
         row     = is_missing.lData[missing_idx] / family_size;
         col     = is_missing.lData[missing_idx] % family_size;
-        fnode   = (col == 0) ? node_id : parents.lData[col-1];
+      //fnode   = (col == 0) ? node_id : parents.lData[col-1];
 		
 		urn = genrand_real2 ();
 		
@@ -941,7 +943,6 @@ _Parameter _BayesianGraphicalModel::ImputeDiscreteNodeScore (long node_id, _Simp
 			row         = is_missing.lData[missing_idx] / family_size;
 			col         = is_missing.lData[missing_idx] % family_size;
 			pa_index    = 0;
-			denom       = 0.;
 			
 			
 			// determine parent combination for this row -- use [pa_indices] object instead?  AFYP
@@ -1114,14 +1115,14 @@ _Parameter _BayesianGraphicalModel::ImputeCGNodeScore (long node_id, _SimpleList
 
     _Parameter      log_score           = 0,
 
-                    impute_maxsteps, impute_burnin, impute_samples, // HBL settings
+ 
+                    parent_state, child_state = 0.0,
 
-                    parent_state, child_state,
-                    denom,
                     // prior hyperparameters for CG nodes
                     rho = prior_sample_size (node_id, 0) > 0 ? (prior_sample_size (node_id, 0) / num_parent_combos) : 1.0,
                     phi = prior_scale (node_id, 0);
 
+    long            impute_maxsteps, impute_burnin, impute_samples; // HBL settings
 
     double          urn;            // uniform random number
 
@@ -1139,9 +1140,9 @@ _Parameter _BayesianGraphicalModel::ImputeCGNodeScore (long node_id, _SimpleList
 	
 
     // set Gibbs sampler parameters from batch language definitions
-    checkParameter (_HYBgm_IMPUTE_MAXSTEPS, impute_maxsteps, 0);
-    checkParameter (_HYBgm_IMPUTE_BURNIN, impute_burnin, 0);
-    checkParameter (_HYBgm_IMPUTE_SAMPLES, impute_samples, 0);
+    checkParameter (_HYBgm_IMPUTE_MAXSTEPS, impute_maxsteps, 0L);
+    checkParameter (_HYBgm_IMPUTE_BURNIN, impute_burnin, 0L);
+    checkParameter (_HYBgm_IMPUTE_SAMPLES, impute_samples, 0L);
 
     if (impute_maxsteps <= 0 || impute_burnin < 0 || impute_samples <= 0 || impute_samples > impute_maxsteps) {
         WarnError (_String("ERROR: Invalid IMPUTE setting(s) in ImputeNodeScore()"));
@@ -1430,7 +1431,6 @@ _Parameter _BayesianGraphicalModel::ImputeCGNodeScore (long node_id, _SimpleList
 			// indices into data_deep_copy for this missing value
 			row         = is_missing.lData[missing_idx] / family_size;
 			col         = is_missing.lData[missing_idx] % family_size;
-			denom       = 0.;
 			pa_index    = pa_indices.lData[row];
 
 
@@ -1589,7 +1589,7 @@ _Parameter _BayesianGraphicalModel::ImputeCGNodeScore (long node_id, _SimpleList
 
 						n_ij.Store (pa_index, 0, n_ij(pa_index,0) - 1);
 						n_ij.Store (pa_indices.lData[row], 0, n_ij(pa_indices.lData[row],0) + 1);
-						pa_index = pa_indices.lData[row];
+            // pa_index = pa_indices.lData[row];
 					}
 				}
 
